/*****************************************************************************
*
*  httpserver.c - General HTTP Server implementation
*  Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*    Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*****************************************************************************/
#include <msp430.h>
#include "wlan.h" 
#include "evnt_handler.h"
#include "nvmem.h"
#include "socket.h"
#include "common.h"
#include "netapp.h"
#include "cc3000.h"
#include "board.h"
#include "string.h"
//#include "uart.h"
#include "demo_config.h"
#include "httpserver.h"
#include "strlib.h"

/** \brief Socket used by server to listen and accept connections */
long httpServerSocket;

/** \brief Variable holding the server's port */
int serverPort = 0;

/** \brief Pointer to the index HTML page */
char * indexPage;
unsigned char dataPacket[5];

/** \brief Pointer to CGI handler structure */
cgi_handler * chList;

/** \brief Pointer to Dynamic HTML handler structure */
dyn_html_handler * htmlList;

/** \brief Pointer to Dynamic HTML handler structure */
http_server_event_handler * eventHandlers = NULL;


/** \brief Client socket handle list */
int clientList[MAX_CLIENTS];

/** \brief Page view counter */
int viewCounter = 1;
#define REQ_BUFFER_SIZE 800
#ifdef __CCS__
#pragma DATA_SECTION(requestBuffer, ".FRAM_DATA")
char requestBuffer[REQ_BUFFER_SIZE];
#elif __IAR_SYSTEMS_ICC__
#pragma location = "FRAM_DATA"
__no_init char requestBuffer[REQ_BUFFER_SIZE];
#endif


extern char OkToDoShutDown;

//*****************************************************************************
//
//! \brief  Initializes HTTP Server
//!
//! \param cnum is the client socket handle to be used
//!
//! \return 0 if successfull
//!
//
//*****************************************************************************
char initHTTPServer(int port, char * ipage,  cgi_handler * handleList, 
                    dyn_html_handler * dhList)
{
    sockaddr serverSocketAddr;
    serverPort = port;
    indexPage = ipage;
    chList = handleList;
    htmlList = dhList;
    
    eventHandlers->serverInitHandlerFunc = NULL;
    eventHandlers->clientAcceptedHandlerFunc = NULL;
    eventHandlers->httpRequestDoneHandlerFunc = NULL;
    
    httpServerSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (httpServerSocket == -1)
    {
        wlan_stop();
        while(1)
        {
            //turnLedOn(CC3000_SOCKET_IND);
            __delay_cycles(100000);
            //turnLedOff(CC3000_SOCKET_IND);
            __delay_cycles(100000);
            
        }
    }
    else
    {
        //turnLedOn(CC3000_SOCKET_IND);
    }
    
    serverSocketAddr.sa_family = AF_INET;
    
    // Set the Port Number
    serverSocketAddr.sa_data[0] = (port & 0xFF00)>> 8;
    serverSocketAddr.sa_data[1] = (port & 0x00FF);

    memset (&serverSocketAddr.sa_data[2], 0, 4);
    
    if (bind(httpServerSocket, &serverSocketAddr, sizeof(sockaddr)) != 0) 
    {
        /*
        while(1)
        {
            turnLedOn(CC3000_SOCKET_IND);
            __delay_cycles(100000);
            turnLedOff(CC3000_SOCKET_IND);
            __delay_cycles(100000);
            
        }*/
    }
	eventHandlers->serverInitHandlerFunc = NULL;
	// Call User Specified Server Initialization Done Event Handler
    /*    
    if(&eventHandlers->serverInitHandlerFunc != NULL)
        {
            eventHandlers->serverInitHandlerFunc(NULL);
        }*/
    
    //
                // Handle any un-solicited event if required - the function shall be triggered 
                // few times in a second
                // 
                hci_unsolicited_event_handler();
                
    turnLedOn(CC3000_SERVER_INIT_IND);
    return 0;
}

//*****************************************************************************
//
//! \brief  Main HTTP Server
//!
//! \param none
//!
//! \return none
//!
//
//*****************************************************************************
void serverMain()
{
    sockaddr clientaddr;  
    socklen_t addrlen;  
    int i = 0;
    int currentClient = 0;
    
    for(i = 0; i < MAX_CLIENTS; i++)
        clientList[i] = -1;
    
    // Start Listening
    if (listen (httpServerSocket, MAX_CLIENTS) != 0)
    {  
        __no_operation();
    }
    
    // Handle Clients and Data
    while(1)
    {
        //
        // Handle any un-solicited event if required - the function shall be triggered 
        // few times in a second
        // 
        hci_unsolicited_event_handler();
        
        addrlen = sizeof(clientaddr);
        
        // accept blocks until we receive a connection
       
        while ( (clientList[currentClient] == -1) || (clientList[currentClient] == -2) )
	{
                 clientList[currentClient] = accept(httpServerSocket, (sockaddr *) &clientaddr, &addrlen);
	}
        //
        // Handle any un-solicited event if required - the function shall be triggered 
        // few times in a second
        // 
        hci_unsolicited_event_handler();
                
        // Call user specified Clietn Accepted Event Handler

        /*
        if(&eventHandlers->clientAcceptedHandlerFunc != NULL)
        {
            eventHandlers->clientAcceptedHandlerFunc(NULL);
        }*/
	
        //turnLedOn(CC3000_HOST_CONNECTED_IND);
        if(clientList[currentClient] >= 0)
        {
            // Connection Accepted, Send Data
            handleHTTPRequest(currentClient);
            closesocket(clientList[currentClient]);
            //turnLedOff(CC3000_HOST_CONNECTED_IND);
            clientList[currentClient]=-1;
            __delay_cycles(1000);
        }
        else if(clientList[currentClient] == -57)
        {
            // BUG: Socket inactive so reopen socket
            // Inactive Socket, close and reopen it
            closesocket(httpServerSocket);
            httpServerSocket = 0xFFFFFFFF;
                
            initHTTPServer(serverPort, indexPage, chList, htmlList);
            
            // Start Listening
            if ( listen (httpServerSocket, 5) != 0 )
            {  
                __no_operation();
            }
        }
        hci_unsolicited_event_handler();
    }
}



//*****************************************************************************
//
//! \brief  Handles HTTP Requests
//!
//! \param cnum is the client socket handle to be used
//!
//! \return none
//!
//
//*****************************************************************************
void handleHTTPRequest(int cnum)
{
    //char requestBuffer[900];
    char * reqline[3];
    char * cgiTok;
    char * delimTok;
    int i = 0;
    char paramBuf[20];
    memset(requestBuffer,0,sizeof (requestBuffer));
    int bytesRecvd;
    bytesRecvd = recv(clientList[cnum], requestBuffer, sizeof(requestBuffer), 0);
    
    if(bytesRecvd > 0) 
    {
        // Received some data, check it and send data back
        reqline[0] = strstr (requestBuffer, "GET");
        if ( reqline[0] != NULL )  
        {
            if (strstr (requestBuffer, "HTTP/1.0") != NULL && 
                strstr (requestBuffer, "HTTP/1.1") != NULL )  
            {  
                send(clientList[cnum], "HTTP/1.0 400 Bad Request\n", 25,0);
            }  
            else  
            {  
                // Currently we only send one file,
                if ( strncmp(reqline[1], "/\0", 2) == 0 )  
                {
                    // We Open index file  (currently no need to handle this)                   
                }
                
#ifdef HTTP_CGI_ENABLED
                // Do we have CGI parameters we need to parse?
                if(strchr(requestBuffer, '?') != NULL)
                {
                    // Decode URL and handle each parameter sequentially 
                    // according to table previously setup.
                    cgiTok = strstr(requestBuffer,"=");
                    if(cgiTok != NULL)
                    {
                        __no_operation();
                        
                        // Handle the 1st parameter
                        delimTok = strstr (cgiTok,"&");
                        memset(paramBuf,0,sizeof(paramBuf));
                        memcpy(paramBuf,cgiTok+1,delimTok-cgiTok-1);
                        chList->cgiHandlerFunc[0](paramBuf);
                        
                        
                        // Handle the rest of the parameters up to the max
                        for(i = 1; i < MAX_CGI_PARAMS; i++)
                        {
                            cgiTok = strtok(NULL, "=");
                            if(cgiTok != NULL)
                            {
                                delimTok = strstr (cgiTok,"&");
                                memset(paramBuf,0,sizeof(paramBuf));
                                memcpy(paramBuf,cgiTok,delimTok-cgiTok);
                                chList->cgiHandlerFunc[i](paramBuf);
                            }                            
                        }    
                    }
                }
#endif                
          
#ifdef HTTP_DYN_HTML_ENABLED
                // The code below replaces data in the HTML page
                // with that generated by the specified functions.
                char tempStr[15];
                for(i = 0; i < 3; i++)
                {
                    memset(tempStr,0,sizeof(tempStr));
                    htmlList->dynHtmlFunc[i](tempStr);
                    tempStr[strlen(tempStr)]= ' ';
                    pageReplace((char *)indexPage, 
                                (char *)htmlList->dynHtmlParamName[i], 
                                (char *)tempStr);
                }                
#endif
                viewCounter++;
                
                sendHTTPData(clientList[cnum], HTTP_RESP, strlen(HTTP_RESP));
                // Send the specificied page in chunks of 300 bytes
                // empirically proven to be sendable
                
                for(i = 0; i < strlen(indexPage); i += 300)
                {
                    if(strlen(indexPage) - i < 300)
                    {
                        sendHTTPData(clientList[cnum], &indexPage[i], 
                                     strlen(indexPage) - i);
                    }
                    else
                    {
                        sendHTTPData(clientList[cnum], &indexPage[i], 300);
                    }                                        
                }   
                // wait untill all packets have been acknowledged to close the socket.
                // The callback function will set the OkToShutDown to 1 once it gets
                // the TX complete event from the CC3000.
                while (1)
                {
                	if  (OkToDoShutDown == 1) break;
                      //__no_operation();
                 }
                // We set the flag back to 0. 
                 OkToDoShutDown = 0;
            }  
        }
        else
        {
            // no GET received. No response
        }
        
        /*
        if(&eventHandlers->httpRequestDoneHandlerFunc != NULL)
        {
            while(1)
            {
                __no_operation();
            }
            eventHandlers->httpRequestDoneHandlerFunc(NULL);
        }*/
    }
    else
    {
        __no_operation();   
    }    
}

//*****************************************************************************
//
//! \brief  Inserts characters in page that appear after an indicator ind
//! with the value from val
//!
//! \param  page is a pointer to the array holding the page's HTML code
//! \param ind is a pointer to a string that has the name of the parameter on the page to modify
//! \param val is the pointer to a string holding the string to insert in the XXX
//!
//! \return none
//!
//
//*****************************************************************************
void pageReplace(char * page, char * ind, char * val)
{
    char * indicLoc;   
    indicLoc = strstr (page,ind);    
    memcpy(indicLoc+strlen(ind), val, strlen(val));
}

//*****************************************************************************
//
//! \brief  Returns a string with the number of views of the page
//!
//! \param  str is a pointer to the array where the number of views will be put
//!
//! \return none
//!
//
//*****************************************************************************
void getViewsNum(char * str)
{     
    itoa(viewCounter,str,10);
}

//*****************************************************************************
//
//! \brief  Sends HTTP Data
//!
//! \param sdesc is the socket descriptor of the socket used for sending data
//! \param buf is a pointer to the buffer with the data to be sent
//! \param len is the number of bytes to send
//!
//! \return none
//!
//
//*****************************************************************************
void sendHTTPData(long sdesc, const void *buf, long len)
{
    int bytesSent = 0;
    bytesSent = send(sdesc, buf, len,0);
    if(bytesSent == -1)
    {
        // General Send Error
    }
    if (bytesSent != strlen(indexPage))
    {
        // ERROR: not all bytes sent
    }
}

//*****************************************************************************
//
//! \brief  Initializes HTTP Server Event Handler
//!
//! \param eh is a pointer to the array handling server events
//!
//! \return none
//!
//
//*****************************************************************************
void initEventHandlers(http_server_event_handler * eh)
{
	eventHandlers = eh;
}
